@@ -103,6 +103,12 @@ static void             contact_list_store_members_changed_cb        (EmpathyCon
 								      gchar                         *message,
 								      gboolean                       is_member,
 								      EmpathyContactListStore       *store);
+static void             contact_list_store_member_renamed_cb         (EmpathyContactList            *list_iface,
+								      EmpathyContact                *old_contact,
+								      EmpathyContact                *new_contact,
+								      guint                          reason,
+								      gchar                         *message,
+								      EmpathyContactListStore       *store);
 static void             contact_list_store_groups_changed_cb         (EmpathyContactList            *list_iface,
 								      EmpathyContact                *contact,
 								      gchar                         *group,
@@ -174,6 +180,10 @@ contact_list_store_iface_setup (gpointer user_data)
 	GList                       *contacts, *l;
 
 	/* Signal connection. */
+	g_signal_connect (priv->list,
+			  "member-renamed",
+			  G_CALLBACK (contact_list_store_member_renamed_cb),
+			  store);
 	g_signal_connect (priv->list,
 			  "members-changed",
 			  G_CALLBACK (contact_list_store_members_changed_cb),
@@ -308,6 +318,9 @@ contact_list_store_dispose (GObject *object)
 	}
 	g_list_free (contacts);
 
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_member_renamed_cb),
+					      object);
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_members_changed_cb),
 					      object);
@@ -789,6 +802,38 @@ contact_list_store_inibit_active_cb (EmpathyContactListStore *store)
 	return FALSE;
 }
 
+static void
+contact_list_store_add_contact_and_connect (EmpathyContactListStore *store, EmpathyContact *contact)
+{
+	g_signal_connect (contact, "notify::presence",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::presence-message",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::name",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::avatar",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::capabilities",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+
+	contact_list_store_add_contact (store, contact);
+}
+
+static void
+contact_list_store_remove_contact_and_disconnect (EmpathyContactListStore *store, EmpathyContact *contact)
+{
+	g_signal_handlers_disconnect_by_func (contact,
+					      G_CALLBACK (contact_list_store_contact_updated_cb),
+					      store);
+
+	contact_list_store_remove_contact (store, contact);
+}
+
 static void
 contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 				       EmpathyContact          *contact,
@@ -808,32 +853,37 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 		is_member ? "added" : "removed");
 
 	if (is_member) {
-		g_signal_connect (contact, "notify::presence",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::presence-message",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::name",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::avatar",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::capabilities",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-
-		contact_list_store_add_contact (store, contact);
+		contact_list_store_add_contact_and_connect (store, contact);
 	} else {
-		g_signal_handlers_disconnect_by_func (contact,
-						      G_CALLBACK (contact_list_store_contact_updated_cb),
-						      store);
-
-		contact_list_store_remove_contact (store, contact);
+		contact_list_store_remove_contact_and_disconnect (store, contact);
 	}
 }
 
+static void
+contact_list_store_member_renamed_cb (EmpathyContactList      *list_iface,
+				      EmpathyContact          *old_contact,
+				      EmpathyContact          *new_contact,
+				      guint                    reason,
+				      gchar                   *message,
+				      EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	priv = GET_PRIV (store);
+
+	DEBUG ("Contact %s (%d) renamed to %s (%d)",
+		empathy_contact_get_id (old_contact),
+		empathy_contact_get_handle (old_contact),
+		empathy_contact_get_id (new_contact),
+		empathy_contact_get_handle (new_contact));
+
+	/* add the new contact */
+	contact_list_store_add_contact_and_connect (store, new_contact);
+
+	/* remove old contact */
+	contact_list_store_remove_contact_and_disconnect (store, old_contact);
+}
+
 static void
 contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 				      EmpathyContact          *contact,
