@@ -110,10 +110,6 @@ typedef struct {
 	TpConnectionPresenceType state;
 	PresenceChooserEntryType previous_type;
 
-	TpConnectionPresenceType   flash_state_1;
-	TpConnectionPresenceType   flash_state_2;
-	guint        flash_timeout_id;
-
 	TpAccountManager *account_manager;
 } EmpathyPresenceChooserPriv;
 
@@ -131,12 +127,6 @@ static struct { TpConnectionPresenceType state;
 
 static void            presence_chooser_finalize               (GObject                    *object);
 static void            presence_chooser_presence_changed_cb    (EmpathyPresenceChooser      *chooser);
-static gboolean        presence_chooser_flash_timeout_cb       (EmpathyPresenceChooser      *chooser);
-static void            presence_chooser_flash_start            (EmpathyPresenceChooser      *chooser,
-								TpConnectionPresenceType                  state_1,
-								TpConnectionPresenceType                  state_2);
-static void            presence_chooser_flash_stop             (EmpathyPresenceChooser      *chooser,
-								TpConnectionPresenceType                  state);
 static void            presence_chooser_menu_add_item          (GtkWidget                  *menu,
 								const gchar                *str,
 								TpConnectionPresenceType                  state);
@@ -282,17 +272,39 @@ presence_chooser_get_entry_type (EmpathyPresenceChooser *self)
 	return type;
 }
 
-static gboolean
-presence_chooser_is_preset (EmpathyPresenceChooser *self)
+static TpConnectionPresenceType
+get_state_and_status (EmpathyPresenceChooser *self,
+	gchar **status)
 {
 	EmpathyPresenceChooserPriv *priv = GET_PRIV (self);
 	TpConnectionPresenceType state;
-	const char *status;
+	gchar *tmp;
+
+	state = tp_account_manager_get_most_available_presence (
+		priv->account_manager, NULL, &tmp);
+	if (EMP_STR_EMPTY (tmp)) {
+		/* no message, use the default message */
+		g_free (tmp);
+		tmp = g_strdup (empathy_presence_get_default_message (state));
+	}
+
+	if (status != NULL)
+		*status = tmp;
+	else
+		g_free (tmp);
+
+	return state;
+}
+
+static gboolean
+presence_chooser_is_preset (EmpathyPresenceChooser *self)
+{
+	TpConnectionPresenceType state;
+	char *status;
 	GList *presets, *l;
 	gboolean match = FALSE;
 
-	state = empathy_idle_get_state (priv->idle);
-	status = empathy_idle_get_status (priv->idle);
+	state = get_state_and_status (self, &status);
 
 	presets = empathy_status_presets_get (state, -1);
 	for (l = presets; l; l = l->next) {
@@ -308,6 +320,7 @@ presence_chooser_is_preset (EmpathyPresenceChooser *self)
 
 	DEBUG ("is_preset(%i, %s) = %i", state, status, match);
 
+	g_free (status);
 	return match;
 }
 
@@ -385,7 +398,7 @@ presence_chooser_set_status_editing (EmpathyPresenceChooser *self,
 
 		/* attempt to get the toplevel for this widget */
 		window = gtk_widget_get_toplevel (GTK_WIDGET (self));
-		if (GTK_WIDGET_TOPLEVEL (window) && GTK_IS_WINDOW (window)) {
+		if (gtk_widget_is_toplevel (window) && GTK_IS_WINDOW (window)) {
 			/* unset the focus */
 			gtk_window_set_focus (GTK_WINDOW (window), NULL);
 		}
@@ -467,15 +480,16 @@ presence_chooser_entry_icon_release_cb (EmpathyPresenceChooser *self,
 	else {
 		PresenceChooserEntryType type;
 		TpConnectionPresenceType state;
-		const char *status;
+		char *status;
 
 		type = presence_chooser_get_entry_type (self);
-		state = empathy_idle_get_state (priv->idle);
-		status = empathy_idle_get_status (priv->idle);
+		state = get_state_and_status (self, &status);
 
-		if (!empathy_status_presets_is_valid (state))
+		if (!empathy_status_presets_is_valid (state)) {
 			/* It doesn't make sense to add such presence as favorite */
+			g_free (status);
 			return;
+		}
 
 		if (presence_chooser_is_preset (self)) {
 			/* remove the entry */
@@ -490,6 +504,7 @@ presence_chooser_entry_icon_release_cb (EmpathyPresenceChooser *self,
 
 		/* update the icon */
 		presence_chooser_set_favorite_icon (self);
+		g_free (status);
 	}
 }
 
@@ -530,7 +545,7 @@ presence_chooser_entry_button_press_event_cb (EmpathyPresenceChooser *self,
 
 	if (!priv->editing_status &&
 	    event->button == 1 &&
-	    !GTK_WIDGET_HAS_FOCUS (entry)) {
+	    !gtk_widget_has_focus (entry)) {
 		gtk_widget_grab_focus (entry);
 		gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
 
@@ -598,7 +613,7 @@ presence_chooser_changed_cb (GtkComboBox *self, gpointer user_data)
 
 		/* attempt to get the toplevel for this widget */
 		window = gtk_widget_get_toplevel (GTK_WIDGET (self));
-		if (!GTK_WIDGET_TOPLEVEL (window) || !GTK_IS_WINDOW (window)) {
+		if (!gtk_widget_is_toplevel (window) || !GTK_IS_WINDOW (window)) {
 			window = NULL;
 		}
 
@@ -619,10 +634,12 @@ presence_chooser_changed_cb (GtkComboBox *self, gpointer user_data)
 		} else {
 			/* else preseed the text of their currently entered
 			 * status message */
-			const char *status;
+			char *status;
 
-			status = empathy_idle_get_status (priv->idle);
+			get_state_and_status (EMPATHY_PRESENCE_CHOOSER (self),
+				&status);
 			gtk_entry_set_text (GTK_ENTRY (entry), status);
+			g_free (status);
 		}
 
 		/* grab the focus */
@@ -749,6 +766,8 @@ update_sensitivity_am_prepared_cb (GObject *source_object,
 		sensitive = FALSE;
 
 	gtk_widget_set_sensitive (GTK_WIDGET (chooser), sensitive);
+
+	presence_chooser_presence_changed_cb (chooser);
 }
 
 static void
@@ -851,12 +870,12 @@ empathy_presence_chooser_init (EmpathyPresenceChooser *chooser)
 			chooser);
 
 	priv->idle = empathy_idle_dup_singleton ();
-	presence_chooser_presence_changed_cb (chooser);
-	g_signal_connect_swapped (priv->idle, "notify::state",
-		G_CALLBACK (presence_chooser_presence_changed_cb),
-		chooser);
 
 	priv->account_manager = tp_account_manager_dup ();
+	g_signal_connect_swapped (priv->account_manager,
+		"most-available-presence-changed",
+		G_CALLBACK (presence_chooser_presence_changed_cb),
+		chooser);
 
 	empathy_signal_connect_weak (priv->account_manager, "account-validity-changed",
 		G_CALLBACK (presence_chooser_account_manager_account_validity_changed_cb),
@@ -891,10 +910,6 @@ presence_chooser_finalize (GObject *object)
 
 	priv = GET_PRIV (object);
 
-	if (priv->flash_timeout_id) {
-		g_source_remove (priv->flash_timeout_id);
-	}
-
 	if (priv->focus_out_idle_source) {
 		g_source_remove (priv->focus_out_idle_source);
 	}
@@ -934,8 +949,7 @@ presence_chooser_presence_changed_cb (EmpathyPresenceChooser *chooser)
 {
 	EmpathyPresenceChooserPriv *priv;
 	TpConnectionPresenceType    state;
-	TpConnectionPresenceType    flash_state;
-	const gchar                *status;
+	gchar                      *status;
 	GtkTreeModel               *model;
 	GtkTreeIter                 iter;
 	gboolean valid, match_state = FALSE, match = FALSE;
@@ -947,9 +961,8 @@ presence_chooser_presence_changed_cb (EmpathyPresenceChooser *chooser)
 		return;
 	}
 
-	priv->state = state = empathy_idle_get_state (priv->idle);
-	status = empathy_idle_get_status (priv->idle);
-	flash_state = empathy_idle_get_flash_state (priv->idle);
+	state = get_state_and_status (chooser, &status);
+	priv->state = state;
 
 	/* An unset presence here doesn't make any sense. Force it to appear as
 	 * offline. */
@@ -1009,82 +1022,16 @@ presence_chooser_presence_changed_cb (EmpathyPresenceChooser *chooser)
 		ui_set_custom_state (chooser, state, status);
 	}
 
-	if (flash_state != TP_CONNECTION_PRESENCE_TYPE_UNSET) {
-		presence_chooser_flash_start (chooser, state, flash_state);
-	}
-	else {
-		presence_chooser_flash_stop (chooser, state);
-	}
+	entry = gtk_bin_get_child (GTK_BIN (chooser));
+	gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
+	      GTK_ENTRY_ICON_PRIMARY,
+	      empathy_icon_name_for_presence (state));
 
 	entry = gtk_bin_get_child (GTK_BIN (chooser));
 	gtk_editable_set_editable (GTK_EDITABLE (entry),
 	    state != TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
-}
 
-static gboolean
-presence_chooser_flash_timeout_cb (EmpathyPresenceChooser *chooser)
-{
-	EmpathyPresenceChooserPriv *priv;
-	TpConnectionPresenceType    state;
-	static gboolean             on = FALSE;
-	GtkWidget                  *entry;
-
-	priv = GET_PRIV (chooser);
-
-	if (on) {
-		state = priv->flash_state_1;
-	}
-	else {
-		state = priv->flash_state_2;
-	}
-
-	entry = gtk_bin_get_child (GTK_BIN (chooser));
-	gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
-					   GTK_ENTRY_ICON_PRIMARY,
-					   empathy_icon_name_for_presence (state));
-
-	on = !on;
-
-	return TRUE;
-}
-
-static void
-presence_chooser_flash_start (EmpathyPresenceChooser *chooser,
-			      TpConnectionPresenceType state_1,
-			      TpConnectionPresenceType state_2)
-{
-	EmpathyPresenceChooserPriv *priv;
-
-	g_return_if_fail (EMPATHY_IS_PRESENCE_CHOOSER (chooser));
-
-	priv = GET_PRIV (chooser);
-
-	priv->flash_state_1 = state_1;
-	priv->flash_state_2 = state_2;
-
-	if (!priv->flash_timeout_id) {
-		priv->flash_timeout_id = g_timeout_add (FLASH_TIMEOUT,
-			(GSourceFunc) presence_chooser_flash_timeout_cb,
-			chooser);
-	}
-}
-
-static void
-presence_chooser_flash_stop (EmpathyPresenceChooser *chooser,
-			     TpConnectionPresenceType state)
-{
-	EmpathyPresenceChooserPriv *priv = GET_PRIV (chooser);
-	GtkWidget *entry;
-
-	if (priv->flash_timeout_id) {
-		g_source_remove (priv->flash_timeout_id);
-		priv->flash_timeout_id = 0;
-	}
-
-	entry = gtk_bin_get_child (GTK_BIN (chooser));
-	gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
-					   GTK_ENTRY_ICON_PRIMARY,
-					   empathy_icon_name_for_presence (state));
+	g_free (status);
 }
 
 /**
