@@ -23,6 +23,7 @@
  *          Martyn Russell <martyn@imendio.com>
  *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  *          Xavier Claessens <xclaesse@gmail.com>
+ *          Rômulo Fernandes Machado <romulo@castorgroup.net>
  */
 
 #include <config.h>
@@ -34,13 +35,13 @@
 #include <glib/gi18n.h>
 #include <libnotify/notification.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-images.h>
@@ -51,10 +52,10 @@
 #include <libempathy-gtk/empathy-smiley-manager.h>
 #include <libempathy-gtk/empathy-sound.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
+#include <libempathy-gtk/empathy-notify-manager.h>
 
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
-#include "empathy-misc.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -72,13 +73,16 @@ typedef struct {
 	GList       *chats_composing;
 	gboolean     page_added;
 	gboolean     dnd_same_window;
-	guint        save_geometry_id;
 	EmpathyChatroomManager *chatroom_manager;
+	EmpathyNotifyManager *notify_mgr;
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
 	NotifyNotification *notification;
 	NotificationData *notification_data;
 
+	GtkTargetList *contact_targets;
+	GtkTargetList *file_targets;
+
 	/* Menu items. */
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
@@ -105,12 +109,22 @@ static const guint tab_accel_keys[] = {
 
 typedef enum {
 	DND_DRAG_TYPE_CONTACT_ID,
+	DND_DRAG_TYPE_URI_LIST,
 	DND_DRAG_TYPE_TAB
 } DndDragType;
 
 static const GtkTargetEntry drag_types_dest[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_TAB },
+	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
+};
+
+static const GtkTargetEntry drag_types_dest_contact[] = {
+	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
+};
+
+static const GtkTargetEntry drag_types_dest_file[] = {
+	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
 };
 
 static void chat_window_update (EmpathyChatWindow *window);
@@ -126,7 +140,7 @@ chat_window_accel_cb (GtkAccelGroup    *accelgroup,
 {
 	EmpathyChatWindowPriv *priv;
 	gint                  num = -1;
-	gint                  i;
+	guint                 i;
 
 	priv = GET_PRIV (window);
 
@@ -267,8 +281,8 @@ chat_window_create_label (EmpathyChatWindow *window,
 		/* We don't want focus/keynav for the button to avoid clutter, and
 		 * Ctrl-W works anyway.
 		 */
-		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
-		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_DEFAULT);
+		gtk_widget_set_can_focus (close_button, FALSE);
+		gtk_widget_set_can_default (close_button, FALSE);
 
 		/* Set the name to make the special rc style match. */
 		gtk_widget_set_name (close_button, "empathy-close-button");
@@ -308,47 +322,41 @@ _submenu_notify_visible_changed_cb (GObject    *object,
 }
 
 static void
-chat_window_update (EmpathyChatWindow *window)
+chat_window_menu_context_update (EmpathyChatWindowPriv *priv,
+			      gint num_pages)
 {
-	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	gboolean               first_page;
-	gboolean               last_page;
-	gboolean               is_connected;
-	gint                   num_pages;
-	gint                   page_num;
-	gint                   i;
-	const gchar           *name;
-	guint                  n_chats;
-	GdkPixbuf             *icon;
-	EmpathyContact        *remote_contact;
-	gboolean               avatar_in_icon;
-	GtkWidget             *chat;
-	GtkWidget             *chat_close_button;
-	GtkWidget             *menu, *submenu, *orig_submenu;
+	gboolean first_page;
+	gboolean last_page;
+	gboolean is_connected;
+	gint     page_num;
 
-	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
-	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
 	first_page = (page_num == 0);
 	last_page = (page_num == (num_pages - 1));
 	is_connected = empathy_chat_get_tp_chat (priv->current_chat) != NULL;
-	name = empathy_chat_get_name (priv->current_chat);
-	n_chats = g_list_length (priv->chats);
 
-	DEBUG ("Update window");
+	DEBUG ("Update window : Menu Contexts (Tabs & Conv)");
 
-	/* Update menu */
-	gtk_action_set_sensitive (priv->menu_tabs_next, !last_page);
-	gtk_action_set_sensitive (priv->menu_tabs_prev, !first_page);
+	gtk_action_set_sensitive (priv->menu_tabs_next, TRUE);
+	gtk_action_set_sensitive (priv->menu_tabs_prev, TRUE);
 	gtk_action_set_sensitive (priv->menu_tabs_detach, num_pages > 1);
 	gtk_action_set_sensitive (priv->menu_tabs_left, !first_page);
 	gtk_action_set_sensitive (priv->menu_tabs_right, !last_page);
 	gtk_action_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
+}
+
+static void
+chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
+				 EmpathyChatWindow     *window)
+{
+	GtkWidget *menu, *submenu, *orig_submenu;
 
-	/* Update Contact menu */
 	menu = gtk_ui_manager_get_widget (priv->ui_manager,
 		"/chats_menubar/menu_contact");
 	orig_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+
+	DEBUG ("Update window : Contact Menu");
+
 	if (orig_submenu == NULL || !GTK_WIDGET_VISIBLE (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
@@ -359,18 +367,113 @@ chat_window_update (EmpathyChatWindow *window)
 					     (GCallback)_submenu_notify_visible_changed_cb,
 					     G_OBJECT (window));
 	}
+}
 
-	/* Update window title */
-	if (n_chats == 1) {
-		gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
-	} else {
-		gchar *title;
+static guint
+get_all_unread_messages (EmpathyChatWindowPriv *priv)
+{
+	GList *l;
+	guint nb = 0;
 
-		title = g_strdup_printf (_("Conversations (%d)"), n_chats);
-		gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
-		g_free (title);
+	for (l = priv->chats_new_msg; l != NULL; l = g_list_next (l)) {
+		EmpathyChat *chat = l->data;
+
+		nb += empathy_chat_get_nb_unread_messages (chat);
 	}
 
+	return nb;
+}
+
+static gchar *
+get_window_title_name (EmpathyChatWindowPriv *priv)
+{
+	const gchar *active_name;
+	guint nb_chats;
+	guint current_unread_msgs;
+
+	nb_chats = g_list_length (priv->chats);
+	g_assert (nb_chats > 0);
+
+	active_name = empathy_chat_get_name (priv->current_chat);
+
+	current_unread_msgs = empathy_chat_get_nb_unread_messages (
+			priv->current_chat);
+
+	if (nb_chats == 1) {
+		/* only one tab */
+		if (current_unread_msgs == 0)
+			return g_strdup (active_name);
+		else
+			return g_strdup_printf (ngettext (
+				"%s (%d unread)",
+				"%s (%d unread)", current_unread_msgs),
+				active_name, current_unread_msgs);
+	} else {
+		guint nb_others = nb_chats - 1;
+		guint all_unread_msgs;
+
+		all_unread_msgs = get_all_unread_messages (priv);
+
+		if (all_unread_msgs == 0) {
+			/* no unread message */
+			return g_strdup_printf (ngettext (
+				"%s (and %u other)",
+				"%s (and %u others)", nb_others),
+				active_name, nb_others);
+		}
+
+		else if (all_unread_msgs == current_unread_msgs) {
+			/* unread messages are in the current tab */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread)",
+				"%s (%d unread)", current_unread_msgs),
+				active_name, current_unread_msgs);
+		}
+
+		else if (current_unread_msgs == 0) {
+			/* unread messages are in other tabs */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread from others)",
+				"%s (%d unread from others)",
+				all_unread_msgs),
+				active_name, all_unread_msgs);
+		}
+
+		else {
+			/* unread messages are in all the tabs */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread from all)",
+				"%s (%d unread from all)",
+				all_unread_msgs),
+				active_name, all_unread_msgs);
+		}
+	}
+}
+
+static void
+chat_window_title_update (EmpathyChatWindowPriv *priv)
+{
+	gchar *name;
+
+	DEBUG ("Update window : Title");
+
+	name = get_window_title_name (priv);
+	gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
+	g_free (name);
+}
+
+static void
+chat_window_icon_update (EmpathyChatWindowPriv *priv)
+{
+	GdkPixbuf      *icon;
+	EmpathyContact *remote_contact;
+	gboolean        avatar_in_icon;
+	guint           n_chats;
+
+	n_chats = g_list_length (priv->chats);
+
+	DEBUG ("Update window : Icon");
+
 	/* Update window icon */
 	if (priv->chats_new_msg) {
 		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
@@ -392,6 +495,17 @@ chat_window_update (EmpathyChatWindow *window)
 			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 		}
 	}
+}
+
+static void
+chat_window_close_button_update (EmpathyChatWindowPriv *priv,
+				 gint num_pages)
+{
+	GtkWidget *chat;
+	GtkWidget *chat_close_button;
+	gint       i;
+
+	DEBUG ("Update window : Close Button");
 
 	if (num_pages == 1) {
 		chat = gtk_notebook_get_nth_page (GTK_NOTEBOOK (priv->notebook), 0);
@@ -408,6 +522,31 @@ chat_window_update (EmpathyChatWindow *window)
 	}
 }
 
+static void
+chat_window_update (EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gint                   num_pages;
+
+	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	DEBUG ("Update window");
+
+	/* Update Tab menu */
+	chat_window_menu_context_update (priv,
+					 num_pages);
+
+	chat_window_contact_menu_update (priv,
+					 window);
+
+	chat_window_title_update (priv);
+
+	chat_window_icon_update (priv);
+
+	chat_window_close_button_update (priv,
+					 num_pages);
+}
+
 static void
 append_markup_printf (GString    *string,
 		      const char *format,
@@ -433,13 +572,15 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	EmpathyContact        *remote_contact;
 	const gchar           *name;
 	const gchar           *id;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	const gchar           *subject;
 	const gchar           *status = NULL;
 	GtkWidget             *widget;
 	GString               *tooltip;
 	gchar                 *markup;
 	const gchar           *icon_name;
+	GtkWidget             *tab_image;
+	GtkWidget             *menu_image;
 
 	window = chat_window_find_chat (chat);
 	if (!window) {
@@ -454,10 +595,14 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	remote_contact = empathy_chat_get_remote_contact (chat);
 
 	DEBUG ("Updating chat tab, name=%s, account=%s, subject=%s, remote_contact=%p",
-		name, empathy_account_get_unique_name (account), subject, remote_contact);
+		name, tp_proxy_get_object_path (account), subject, remote_contact);
 
 	/* Update tab image */
-	if (g_list_find (priv->chats_new_msg, chat)) {
+	if (empathy_chat_get_tp_chat (chat) == NULL) {
+		/* No TpChat, we are disconnected */
+		icon_name = NULL;
+	}
+	else if (g_list_find (priv->chats_new_msg, chat)) {
 		icon_name = EMPATHY_IMAGE_MESSAGE;
 	}
 	else if (g_list_find (priv->chats_composing, chat)) {
@@ -468,10 +613,18 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	} else {
 		icon_name = EMPATHY_IMAGE_GROUP_MESSAGE;
 	}
-	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
-	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_MENU);
-	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-image");
-	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_MENU);
+
+	tab_image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
+	menu_image = g_object_get_data (G_OBJECT (chat), "chat-window-menu-image");
+	if (icon_name != NULL) {
+		gtk_image_set_from_icon_name (GTK_IMAGE (tab_image), icon_name, GTK_ICON_SIZE_MENU);
+		gtk_widget_show (tab_image);
+		gtk_image_set_from_icon_name (GTK_IMAGE (menu_image), icon_name, GTK_ICON_SIZE_MENU);
+		gtk_widget_show (menu_image);
+	} else {
+		gtk_widget_hide (tab_image);
+		gtk_widget_hide (menu_image);
+	}
 
 	/* Update tab tooltip */
 	tooltip = g_string_new (NULL);
@@ -486,7 +639,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	append_markup_printf (tooltip,
 			      "<b>%s</b><small> (%s)</small>",
 			      id,
-			      empathy_account_get_display_name (account));
+			      tp_account_get_display_name (account));
 
 	if (!EMP_STR_EMPTY (status)) {
 		append_markup_printf (tooltip, "\n<i>%s</i>", status);
@@ -581,7 +734,7 @@ chat_window_conv_activate_cb (GtkAction         *action,
 	is_room = empathy_chat_is_room (priv->current_chat);
 	if (is_room) {
 		const gchar *room;
-		EmpathyAccount   *account;
+		TpAccount   *account;
 		gboolean     found = FALSE;
 		EmpathyChatroom *chatroom;
 
@@ -630,7 +783,7 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               active;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
@@ -648,7 +801,7 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 		chatroom = empathy_chatroom_new_full (account, room, name, FALSE);
 		empathy_chatroom_manager_add (priv->chatroom_manager, chatroom);
 		g_object_unref (chatroom);
-  }
+	}
 
 	empathy_chatroom_set_favorite (chatroom, active);
 }
@@ -665,63 +818,6 @@ chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
 	empathy_chat_set_show_contacts (priv->current_chat, active);
 }
 
-static const gchar *
-chat_get_window_id_for_geometry (EmpathyChat *chat)
-{
-	const gchar *res = NULL;
-	gboolean     separate_windows;
-
-	empathy_conf_get_bool (empathy_conf_get (),
-			       EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
-			       &separate_windows);
-
-	if (separate_windows) {
-		res = empathy_chat_get_id (chat);
-	}
-
-	return res ? res : "chat-window";
-}
-
-static gboolean
-chat_window_save_geometry_timeout_cb (EmpathyChatWindow *window)
-{
-	EmpathyChatWindowPriv *priv;
-	gint                  x, y, w, h;
-
-	priv = GET_PRIV (window);
-
-	gtk_window_get_size (GTK_WINDOW (priv->dialog), &w, &h);
-	gtk_window_get_position (GTK_WINDOW (priv->dialog), &x, &y);
-
-	empathy_geometry_save (chat_get_window_id_for_geometry (priv->current_chat),
-			       x, y, w, h);
-
-	priv->save_geometry_id = 0;
-
-	return FALSE;
-}
-
-static gboolean
-chat_window_configure_event_cb (GtkWidget         *widget,
-				GdkEventConfigure *event,
-				EmpathyChatWindow  *window)
-{
-	EmpathyChatWindowPriv *priv;
-
-	priv = GET_PRIV (window);
-
-	if (priv->save_geometry_id != 0) {
-		g_source_remove (priv->save_geometry_id);
-	}
-
-	priv->save_geometry_id =
-		g_timeout_add_seconds (1,
-				       (GSourceFunc) chat_window_save_geometry_timeout_cb,
-				       window);
-
-	return FALSE;
-}
-
 static void
 chat_window_close_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
@@ -812,25 +908,69 @@ chat_window_paste_activate_cb (GtkAction         *action,
 	empathy_chat_paste (priv->current_chat);
 }
 
+static void
+chat_window_tabs_next_activate_cb (GtkAction         *action,
+				   EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+	EmpathyChat           *chat;
+	gint                  index_, numPages;
+
+	priv = GET_PRIV (window);
+
+	chat = priv->current_chat;
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	if (index_ == (numPages - 1)) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
+		return;
+	}
+
+	gtk_notebook_next_page (GTK_NOTEBOOK (priv->notebook));
+}
+
+static void
+chat_window_tabs_previous_activate_cb (GtkAction         *action,
+				   EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+	EmpathyChat           *chat;
+	gint                  index_, numPages;
+
+	priv = GET_PRIV (window);
+
+	chat = priv->current_chat;
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	if (index_ <= 0) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), numPages - 1);
+		return;
+	}
+
+	gtk_notebook_prev_page (GTK_NOTEBOOK (priv->notebook));
+}
+
 static void
 chat_window_tabs_left_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
 	EmpathyChat           *chat;
-	gint                  index;
+	gint                  index_;
 
 	priv = GET_PRIV (window);
 
 	chat = priv->current_chat;
-	index = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
-	if (index <= 0) {
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	if (index_ <= 0) {
 		return;
 	}
 
 	gtk_notebook_reorder_child (GTK_NOTEBOOK (priv->notebook),
 				    GTK_WIDGET (chat),
-				    index - 1);
+				    index_ - 1);
 }
 
 static void
@@ -839,16 +979,16 @@ chat_window_tabs_right_activate_cb (GtkAction         *action,
 {
 	EmpathyChatWindowPriv *priv;
 	EmpathyChat           *chat;
-	gint                  index;
+	gint                  index_;
 
 	priv = GET_PRIV (window);
 
 	chat = priv->current_chat;
-	index = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 
 	gtk_notebook_reorder_child (GTK_NOTEBOOK (priv->notebook),
 				    GTK_WIDGET (chat),
-				    index + 1);
+				    index_ + 1);
 }
 
 static void
@@ -976,7 +1116,7 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean res;
 
-	if (!empathy_notification_is_enabled ()) {
+	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		return;
 	} else {
 		empathy_conf_get_bool (empathy_conf_get (),
@@ -1008,7 +1148,8 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
 	}
 
-	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
+	pixbuf = empathy_notify_manager_get_pixbuf_for_notification (priv->notify_mgr,
+		sender, EMPATHY_IMAGE_NEW_MESSAGE);
 
 	if (pixbuf != NULL) {
 		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
@@ -1067,6 +1208,8 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	}
 
 	if (has_focus && priv->current_chat == chat) {
+		/* window and tab are focused so consider the message to be read */
+		empathy_chat_messages_read (chat);
 		return;
 	}
 
@@ -1097,6 +1240,9 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 		    EMPATHY_SOUND_MESSAGE_INCOMING);
 		chat_window_show_or_update_notification (window, message, chat);
 	}
+
+	/* update the number of unread messages */
+	chat_window_title_update (priv);
 }
 
 static GtkNotebook *
@@ -1153,6 +1299,7 @@ chat_window_page_switched_cb (GtkNotebook      *notebook,
 
 	priv->current_chat = chat;
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+	empathy_chat_messages_read (chat);
 
 	chat_window_update_chat_tab (chat);
 }
@@ -1190,6 +1337,9 @@ chat_window_page_added_cb (GtkNotebook      *notebook,
 	g_signal_connect (chat, "new-message",
 			  G_CALLBACK (chat_window_new_message_cb),
 			  window);
+	g_signal_connect (chat, "notify::tp-chat",
+			  G_CALLBACK (chat_window_update_chat_tab),
+			  window);
 
 	/* Set flag so we know to perform some special operations on
 	 * switch page due to the new page being added.
@@ -1234,10 +1384,14 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 	g_signal_handlers_disconnect_by_func (chat,
 					      G_CALLBACK (chat_window_new_message_cb),
 					      window);
+	g_signal_handlers_disconnect_by_func (chat,
+					      G_CALLBACK (chat_window_update_chat_tab),
+					      window);
 
 	/* Keep list of chats up to date */
 	priv->chats = g_list_remove (priv->chats, chat);
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+	empathy_chat_messages_read (chat);
 	priv->chats_composing = g_list_remove (priv->chats_composing, chat);
 
 	if (priv->chats == NULL) {
@@ -1259,6 +1413,7 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	priv = GET_PRIV (window);
 
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, priv->current_chat);
+	empathy_chat_messages_read (priv->current_chat);
 
 	chat_window_set_urgency_hint (window, FALSE);
 
@@ -1268,6 +1423,68 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	return FALSE;
 }
 
+static gboolean
+chat_window_drag_motion (GtkWidget        *widget,
+			 GdkDragContext   *context,
+			 int               x,
+			 int               y,
+			 guint             time_,
+			 EmpathyChatWindow *window)
+{
+	GdkAtom target;
+	EmpathyChatWindowPriv *priv;
+	GdkAtom dest_target;
+
+	priv = GET_PRIV (window);
+
+	target = gtk_drag_dest_find_target (widget, context, NULL);
+
+	dest_target = gdk_atom_intern_static_string ("text/uri-list");
+	if (target == dest_target) {
+		/* This is a file drag.  Ensure the contact is online and set the
+		   drag type to COPY.  Note that it's possible that the tab will
+		   be switched by GTK+ after a timeout from drag_motion without
+		   getting another drag_motion to disable the drop.  You have
+		   to hold your mouse really still.
+		 */
+		EmpathyContact *contact;
+
+		priv = GET_PRIV (window);
+		contact = empathy_chat_get_remote_contact (priv->current_chat);
+		/* contact is NULL for multi-user chats.  We don't do
+		 * file transfers to MUCs.  We also don't send files
+		 * to offline contacts or contacts that don't support
+		 * file transfer.
+		 */
+		if ((contact == NULL) || !empathy_contact_is_online (contact)) {
+			gdk_drag_status (context, 0, time_);
+			return FALSE;
+		}
+		if (!(empathy_contact_get_capabilities (contact)
+			   & EMPATHY_CAPABILITIES_FT)) {
+			gdk_drag_status (context, 0, time_);
+			return FALSE;
+		}
+		gdk_drag_status (context, GDK_ACTION_COPY, time_);
+		return TRUE;
+	}
+
+	dest_target = gdk_atom_intern_static_string ("text/contact-id");
+	if (target == dest_target) {
+		/* This is a drag of a contact from a contact list.  Set to COPY.
+		   FIXME: If this drag is to a MUC window, it invites the user.
+		   Otherwise, it opens a chat.  Should we use a different drag
+		   type for invites?  Should we allow ASK?
+		 */
+		gdk_drag_status (context, GDK_ACTION_COPY, time_);
+		return TRUE;
+	}
+
+	/* Otherwise, it must be a notebook tab drag.  Set to MOVE. */
+	gdk_drag_status (context, GDK_ACTION_MOVE, time_);
+	return TRUE;
+}
+
 static void
 chat_window_drag_data_received (GtkWidget        *widget,
 				GdkDragContext   *context,
@@ -1275,21 +1492,24 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				int               y,
 				GtkSelectionData *selection,
 				guint             info,
-				guint             time,
+				guint             time_,
 				EmpathyChatWindow *window)
 {
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
 		EmpathyChat           *chat = NULL;
 		EmpathyChatWindow     *old_window;
-		EmpathyAccount        *account = NULL;
-		EmpathyAccountManager *account_manager;
+		TpAccount             *account = NULL;
+		TpAccountManager      *account_manager;
 		const gchar           *id;
 		gchar                **strv;
 		const gchar           *account_id;
 		const gchar           *contact_id;
 
 		id = (const gchar*) gtk_selection_data_get_data (selection);
-		account_manager = empathy_account_manager_dup_singleton ();
+
+		/* FIXME: Perhaps should be sure that the account manager is
+		 * prepared before calling _ensure_account on it. */
+		account_manager = tp_account_manager_dup ();
 
 		DEBUG ("DND contact from roster with id:'%s'", id);
 
@@ -1298,21 +1518,21 @@ chat_window_drag_data_received (GtkWidget        *widget,
 			account_id = strv[0];
 			contact_id = strv[1];
 			account =
-				empathy_account_manager_get_account (account_manager, account_id);
+				tp_account_manager_ensure_account (account_manager, account_id);
 			if (account != NULL)
 				chat = empathy_chat_window_find_chat (account, contact_id);
 		}
 
 		if (account == NULL) {
 			g_strfreev (strv);
-			gtk_drag_finish (context, FALSE, FALSE, time);
+			gtk_drag_finish (context, FALSE, FALSE, time_);
 			return;
 		}
 
 		if (!chat) {
 			TpConnection *connection;
 
-			connection = empathy_account_get_connection (account);
+			connection = tp_account_get_connection (account);
 
 			if (connection) {
 				empathy_dispatcher_chat_with_contact_id (
@@ -1328,7 +1548,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		old_window = chat_window_find_chat (chat);
 		if (old_window) {
 			if (old_window == window) {
-				gtk_drag_finish (context, TRUE, FALSE, time);
+				gtk_drag_finish (context, TRUE, FALSE, time_);
 				return;
 			}
 
@@ -1345,7 +1565,28 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 * weird consequences, and we handle that internally
 		 * anyway with add_chat () and remove_chat ().
 		 */
-		gtk_drag_finish (context, TRUE, FALSE, time);
+		gtk_drag_finish (context, TRUE, FALSE, time_);
+	}
+	else if (info == DND_DRAG_TYPE_URI_LIST) {
+		EmpathyChatWindowPriv *priv;
+		EmpathyContact *contact;
+		const gchar *data;
+
+		priv = GET_PRIV (window);
+		contact = empathy_chat_get_remote_contact (priv->current_chat);
+
+		/* contact is NULL when current_chat is a multi-user chat.
+		 * We don't do file transfers to MUCs, so just cancel the drag.
+		 */
+		if (contact == NULL) {
+			gtk_drag_finish (context, TRUE, FALSE, time_);
+			return;
+		}
+
+		data = (const gchar *) gtk_selection_data_get_data (selection);
+		empathy_send_file_from_uri_list (contact, data);
+
+		gtk_drag_finish (context, TRUE, FALSE, time_);
 	}
 	else if (info == DND_DRAG_TYPE_TAB) {
 		EmpathyChat        **chat;
@@ -1364,7 +1605,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 			if (old_window == window) {
 				DEBUG ("DND tab (within same window)");
 				priv->dnd_same_window = TRUE;
-				gtk_drag_finish (context, TRUE, FALSE, time);
+				gtk_drag_finish (context, TRUE, FALSE, time_);
 				return;
 			}
 
@@ -1376,10 +1617,10 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 * weird consequences, and we handle that internally
 		 * anyway with add_chat () and remove_chat ().
 		 */
-		gtk_drag_finish (context, TRUE, FALSE, time);
+		gtk_drag_finish (context, TRUE, FALSE, time_);
 	} else {
 		DEBUG ("DND from unknown source");
-		gtk_drag_finish (context, FALSE, FALSE, time);
+		gtk_drag_finish (context, FALSE, FALSE, time_);
 	}
 }
 
@@ -1396,10 +1637,7 @@ chat_window_finalize (GObject *object)
 
 	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->chatroom_manager);
-	if (priv->save_geometry_id != 0) {
-		g_source_remove (priv->save_geometry_id);
-		chat_window_save_geometry_timeout_cb (window);
-	}
+	g_object_unref (priv->notify_mgr);
 
 	if (priv->notification != NULL) {
 		notify_notification_close (priv->notification, NULL);
@@ -1412,6 +1650,13 @@ chat_window_finalize (GObject *object)
 			}
 	}
 
+	if (priv->contact_targets) {
+		gtk_target_list_unref (priv->contact_targets);
+	}
+	if (priv->file_targets) {
+		gtk_target_list_unref (priv->file_targets);
+	}
+
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
@@ -1448,7 +1693,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	GClosure              *closure;
 	GtkWidget             *menu;
 	GtkWidget             *submenu;
-	gint                   i;
+	guint                  i;
 	GtkWidget             *chat_vbox;
 	gchar                 *filename;
 	EmpathySmileyManager  *smiley_manager;
@@ -1476,7 +1721,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	g_free (filename);
 
 	empathy_builder_connect (gui, window,
-			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
@@ -1486,6 +1730,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
 			      "menu_edit_copy", "activate", chat_window_copy_activate_cb,
 			      "menu_edit_paste", "activate", chat_window_paste_activate_cb,
+			      "menu_tabs_next", "activate", chat_window_tabs_next_activate_cb,
+			      "menu_tabs_prev", "activate", chat_window_tabs_previous_activate_cb,
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
 			      "menu_tabs_right", "activate", chat_window_tabs_right_activate_cb,
 			      "menu_tabs_detach", "activate", chat_window_detach_activate_cb,
@@ -1522,6 +1768,12 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	g_object_unref (accel_group);
 
+	/* Set up drag target lists */
+	priv->contact_targets = gtk_target_list_new (drag_types_dest_contact,
+						     G_N_ELEMENTS (drag_types_dest_contact));
+	priv->file_targets = gtk_target_list_new (drag_types_dest_file,
+						  G_N_ELEMENTS (drag_types_dest_file));
+
 	/* Set up smiley menu */
 	smiley_manager = empathy_smiley_manager_dup_singleton ();
 	submenu = empathy_smiley_menu_new (smiley_manager,
@@ -1540,16 +1792,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			  "delete_event",
 			  G_CALLBACK (chat_window_delete_event_cb),
 			  window);
-
-	g_signal_connect_swapped (priv->menu_tabs_prev,
-				  "activate",
-				  G_CALLBACK (gtk_notebook_prev_page),
-				  priv->notebook);
-	g_signal_connect_swapped (priv->menu_tabs_next,
-				  "activate",
-				  G_CALLBACK (gtk_notebook_next_page),
-				  priv->notebook);
-
 	g_signal_connect (priv->dialog,
 			  "focus_in_event",
 			  G_CALLBACK (chat_window_focus_in_event_cb),
@@ -1572,8 +1814,13 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			   GTK_DEST_DEFAULT_ALL,
 			   drag_types_dest,
 			   G_N_ELEMENTS (drag_types_dest),
-			   GDK_ACTION_MOVE);
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
+	/* connect_after to allow GtkNotebook's built-in tab switching */
+	g_signal_connect_after (priv->notebook,
+				"drag-motion",
+				G_CALLBACK (chat_window_drag_motion),
+				window);
 	g_signal_connect (priv->notebook,
 			  "drag-data-received",
 			  G_CALLBACK (chat_window_drag_data_received),
@@ -1586,6 +1833,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	priv->chats_new_msg = NULL;
 	priv->chats_composing = NULL;
 	priv->current_chat = NULL;
+
+	priv->notify_mgr = empathy_notify_manager_dup_singleton ();
 }
 
 EmpathyChatWindow *
@@ -1649,7 +1898,6 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	GtkWidget             *label;
 	GtkWidget             *popup_label;
 	GtkWidget             *child;
-	gint                   x, y, w, h;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -1661,21 +1909,18 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 
 	/* If this window has just been created, position it */
 	if (priv->chats == NULL) {
-		empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
+		const gchar *name = "chat-window";
+		gboolean     separate_windows;
 
-		if (x >= 0 && y >= 0) {
-			/* Let the window manager position it if we don't have
-			 * good x, y coordinates.
-			 */
-			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
+				       &separate_windows);
+
+		if (separate_windows) {
+			name = empathy_chat_get_id (chat);
 		}
 
-		if (w > 0 && h > 0) {
-			/* Use the defaults from the ui file if we don't have
-			 * good w, h geometry.
-			 */
-			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
-		}
+		empathy_geometry_bind (GTK_WINDOW (priv->dialog), name);
 	}
 
 	child = GTK_WIDGET (chat);
@@ -1799,7 +2044,7 @@ empathy_chat_window_has_focus (EmpathyChatWindow *window)
 }
 
 EmpathyChat *
-empathy_chat_window_find_chat (EmpathyAccount   *account,
+empathy_chat_window_find_chat (TpAccount   *account,
 			       const gchar *id)
 {
 	GList *l;
