@@ -24,11 +24,12 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/interfaces.h>
 
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
@@ -337,14 +338,14 @@ event_channel_process_voip_func (EventPriv *event)
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
       _("_Reject"), GTK_RESPONSE_REJECT);
-  image = gtk_image_new_from_icon_name (GTK_STOCK_CANCEL,
+  image = gtk_image_new_from_icon_name ("call-stop",
     GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
       _("_Answer"), GTK_RESPONSE_ACCEPT);
 
-  image = gtk_image_new_from_icon_name (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name ("call-start", GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   g_signal_connect (dialog, "response",
@@ -713,7 +714,7 @@ event_manager_ft_got_contact_cb (EmpathyTpContactFactory *factory,
   EventManagerApproval *approval = (EventManagerApproval *) user_data;
   char *header;
 
-  approval->contact = contact;
+  approval->contact = g_object_ref (contact);
 
   header = g_strdup_printf (_("Incoming file transfer from %s"),
                             empathy_contact_get_name (approval->contact));
@@ -918,13 +919,16 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
     TpConnectionPresenceType previous,
     EmpathyEventManager *manager)
 {
-  EmpathyAccount *account;
+  TpAccount *account;
   gchar *header = NULL;
   gboolean preference = FALSE;
+  EmpathyIdle *idle;
 
   account = empathy_contact_get_account (contact);
-  if (empathy_account_is_just_connected (account))
-    return;
+  idle = empathy_idle_dup_singleton ();
+
+  if (empathy_idle_account_is_just_connected (idle, account))
+    goto out;
 
   if (tp_connection_presence_type_cmp_availability (previous,
      TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
@@ -940,7 +944,7 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
             empathy_contact_get_name (contact));
 
           event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-              GTK_STOCK_DIALOG_INFO, header, NULL, NULL, NULL, NULL);
+              "stock_person", header, NULL, NULL, NULL, NULL);
         }
     }
   else
@@ -956,10 +960,13 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
             empathy_contact_get_name (contact));
 
           event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-              GTK_STOCK_DIALOG_INFO, header, NULL, NULL, NULL, NULL);
+              "stock_person", header, NULL, NULL, NULL, NULL);
         }
     }
   g_free (header);
+
+out:
+  g_object_unref (idle);
 }
 
 
